// ****************** MUST INCLUDE LINES BELOW*****************************
#include <lib6302.h>
// Init library and set pwm freq, DAC resolution, ADC resolution, and teensy 3.6-3.2 bit rate.
// PWM frequency affects DAC resolution, higher frequency, lower resolution.
// For 8-bit PWM: Teensy 3.6-234375, Teensy 3.2-187500!!!1
// Divide by 2,4,8 or 16 for 9,10,11, or 12 bits.
// WARNING 12 bit pwm is in audio range for young listeners.
#define MYSERIAL Serial
lib6302 my6302(234375/8,12,16,&Serial);
// Set up the analog-to-digital library, with a buffersize
#define sampleBufN 500
LIB6302_ADC(sampleBufN+100);  // ADC sample buffer of 100+sampleBufN
// ****************** MUST INCLUDE LINES ABOVE*******1**********************

// Constants for Users to Modify 
// ************************************************************
// MUST SET CALIBRATION COEFFIFICENTS!!!
// motor, theta (angle), and backEMF calibration.
// Scale, Offset, and nomCmd MUST BE CONSISTENT WITH 6302_Obs_21!
#define thetaOffset 0.06  // Adjust if measured angle is not zero when horizontal. 
#define nomCmdL 0.5 // Adjust to zero steady-state error
#define nomCmdR 0.5 // Adjust to zero steady-state error

#define default_Ampl 0.1 // Sweeper default amplitude
#define KP 0.5
#define KD 0.03
#define mBack 3
#define NUMREP 1

#define controllerUpdatePeriod 1000 // Update period in microseconds

// Frequency Sweep Parameters
#define firstFreq 0.3
#define lastFreq 3.0

// Variables for Conversion
#define deg2rad 0.0175
#define TWOPI 6.2831853
#define MAX_FREQ_PTS 30

// ************************************************************************
// End of Constants and code for Users to Modify 
// SEE END OF FILE FOR USER-ALTERABLE CONTROL LOOP!!

// Sweeper variables
float freqs[MAX_FREQ_PTS];
float Fmultiplier;
float newFreq;
boolean newFreqFlg;
int freqIndex;
int periodInDts;
int numDts, numPeriods;
float sa,ca, sc,cc, sd,cd;


// *******SETTING UP THE MONITOR DISPLAY (SLIDERS AND PLOTS). *************
// Sliders
String sliders = "S~Pause~0~1~";
#define PA 0

// Plots: USE SPACES BETWEEN LABELS FOR MULTIPLOTS!!
String freqP = "P~Freq(Hz)~-0~1~90~1~";
String magP = "P~Magnitude~-0.1~3.0~90~1~";
String phaseP = "P~Phase(Deg)~-3~0.1~90~1~";
String magCmdP = "P~MagCmd~-0.1~3.0.0~90~1~";
String phaseCmdP = "P~PhaseCmd(Deg)~-2~2~90~1~";
String monPlots = freqP + magP + phaseP + magCmdP + phaseCmdP;
#define PLOT_F 0
#define PLOT_MAGA 1
#define PLOT_PHASEA 2
#define PLOT_MAGC 3
#define PLOT_PHASEC 4

// Configuration string with surrounding limits.
String config_message = sliders + monPlots;

// Control loop setup
// ************************************************************************

#define twopi 6.2831853

float deltaTSecs = 0.0;
float tenSecInDt = 0.0;
float mDt = 0.0;
float uMax = 0.0;

elapsedMicros loopTimer; // loopTime used to force precise deltaT between starts
float headRoom = float(controllerUpdatePeriod);
uint32_t loopCntr = 0;

#define baudRate 1000000

void setup() {
  MYSERIAL.begin(baudRate);
  my6302.setup(config_message, 1.0);
  my6302.startPWM(true); // start parallel drive pwm, A+B in sync.
  my6302.setupADC(16,false);
  loopTimer = 0;
  loopCntr = 0;
  deltaTSecs = float(controllerUpdatePeriod)*1.0e-6;
  mDt = mBack*deltaTSecs;
  headRoom = controllerUpdatePeriod;
  tenSecInDt = int(10.0/deltaTSecs);

  // Sweeper setups
  float nomCmdMin = min(nomCmdL,nomCmdR);
  float nomCmdMax = max(nomCmdL,nomCmdR);
  uMax = min(nomCmdMin, 1-nomCmdMax);
  
  freqIndex = 0;
  newFreqFlg = true;
  newFreq = firstFreq;
  Fmultiplier = exp(log(lastFreq/firstFreq)/float(MAX_FREQ_PTS));
  
  digitalWrite(nSleepPWM, HIGH);  // Turn on PWM driver
}
// Motor PWM and Sensor defines.
float oldTheta_d = 0.0;
int plotCycle = 0;

float cosv = 0.0;
float sinv = 0.0;

float updateThetaDsweeper() {

    if(!newFreqFlg) numDts += 1; 
    else { // Initialize next freq pt, period MUST be integral multiple of deltaT
      newFreqFlg = false;
      periodInDts = 2*int(0.5/(deltaTSecs*newFreq));
      numPeriods = 0;
      if( (periodInDts < 10) || (freqIndex == MAX_FREQ_PTS) ) { // new Freq too high, restort
        newFreq = firstFreq;
        freqIndex = 0; 
        periodInDts = 2*int(0.5/(deltaTSecs*newFreq));
      }
      newFreq = float(1.0)/(deltaTSecs*float(periodInDts)); // Force period = N*deltaT 
      numDts = 0; sc = 0; cc = 0; sa = 0; ca = 0; sd = 0; cd = 0; 
    }
    float arg = (twopi*float(numDts))/float(periodInDts);
    cosv = cos(arg);
    sinv = sin(arg);
  
    // Compute next sine wave point, send to analog out
    return(default_Ampl*sinv);
}

float update_status_sweeper(float u, float theta, float theta_d) {

  if(numDts < periodInDts) { // Not on period boundary, accumulate inintegral.
    sc += u*sinv; cc += u*cosv; 
    sa += theta*sinv; ca += theta*cosv;
    sd += theta_d*sinv; cd += theta_d*cosv;
  } else { // On a period boundary.
    numPeriods += 1;
    if ( (numPeriods > 3)  &&  numPeriods*periodInDts > tenSecInDt) {
       newFreqFlg = true;  // more then 2 periods AND longer than 10 secs, new freq.
    } else {
       numDts = 0; sa = 0.0; ca = 0.0; sc = 0.0; cc = 0.0; sd = 0.0; cd = 0.0;
    }
  }

  if(newFreqFlg) {  //  Just finished a frequency, dump out results
    float phaseSys = atan2(ca,sa)/deg2rad;
    if(phaseSys > 0) phaseSys -= 360;
    float phaseCmd = atan2(cc,sc)/deg2rad;
    if(phaseCmd > 180) phaseCmd -= 360;
    float magD = sqrt(sd*sd + cd*cd);    
    float magC = sqrt(sc*sc + cc*cc)/magD;
    float magA = sqrt(sa*sa + ca*ca)/magD;

    if(!my6302.isFirst()) {
      float loopStatus[6];
      loopStatus[PLOT_F] = newFreq;
      loopStatus[PLOT_MAGA] = magA;
      loopStatus[PLOT_PHASEA] = phaseSys;
      loopStatus[PLOT_MAGC] = magC;
      loopStatus[PLOT_PHASEC] = phaseCmd;
      for(int i=0; i < NUMREP; i++) {
        my6302.sendStatus(5,loopStatus);
      }
    }
    newFreq *= Fmultiplier;
    freqIndex += 1;
    headRoom = controllerUpdatePeriod;
  } 
  return(theta);
}

#define zeroParamThresh 0.005
#define scaleParam 1.005
float analog2param(int pinA) {
  float retval = my6302.adcGetAvg(pinA) - zeroParamThresh;
  retval = scaleParam * max(retval, 0.0);
  return(retval);
}

//********************** Main USER ALTERABLE Control Loop **********************
//**************************************************************************
float pastTheta[mBack+1];  // Stores past errors for deriv approx

void loop() {  

  // Initialize: check for inputs, flip toggle, etc.
  my6302.startloop();

  // Get parameters.
  float pauseVal= my6302.getSlider(PA);

  // Headroom is faction of period left until next sensor read.
  float headRoomThis = float(controllerUpdatePeriod - int(loopTimer));
  headRoom = headRoomThis;
  
  // Wait exactly one deltaT since last sensor read.
  while (int(loopTimer) < controllerUpdatePeriod) {};
  loopTimer = 0;

  // Get theta_d, theta_a, omega, and updated saved thetas.
  float theta_a = TWOPI*(0.5 - my6302.adcGetAvg(A1)) + thetaOffset;
  float omega_a = (theta_a - pastTheta[mBack])/mDt;
  float theta_d = updateThetaDsweeper(); 
  float u = KP*(theta_d - theta_a) - KD*omega_a;
  float uClip = min(uMax,max(-uMax,u));
  
  // Save backward angles for delta/deltaT calculation.
  for (int i = mBack; i > 0; i--) pastTheta[i] = pastTheta[i-1];
  pastTheta[0] = theta_a;

  // Write motor command Hbridges, return clipped value.
  my6302.hbridgeWrite(nomCmdL + uClip, pwmB1); // B1->B2 flips motorndir
  my6302.hbridgeWrite(nomCmdR - uClip, pwmB2_T); // B1_T->B2_T flips motor dir
  
  update_status_sweeper(uClip, theta_a, theta_d);
}




  


  
