function DTSim(Ad,Bd,Cd,Krd,Kd,Ld,DeltaT,Bi,NoiseAmp,perturb,stateNames)

% =========
% Discrete-Time Simulation
% =========

if(max(abs(eig(Ad-Bd*Kd))) >= 1.0)
    disp('System not stable!!, No simulation performed');
    return
end

% Determine the number of states.
numStates = size(Ad,1); % Number of states equals Ad's # rows.

% 2000 steps or enough so that max(abs(natfreqs))^numsteps < 0.01.
numSteps = 2*max(1000,floor(log(0.01)/log(max(abs(eig(Ad-Bd*Kd))))));
t = (0:numSteps-1)*DeltaT;

% Set y_desired to be a step input, changing from 0
y_desired = zeros(numSteps,1);
stepChangeIndex = floor(numSteps/2);
y_desired(stepChangeIndex:end) = 1.0;
y_desired(stepChangeIndex+1:stepChangeIndex+10) = 0.1*(1:10);

% Give the step a slight ramp (makes better plots).
y_desired(stepChangeIndex+1:stepChangeIndex+10) = 0.1*(1:10);

% Simulate and Save Three Sets of States:
%  1) Estimated state, 
%  2) Actual state when using estimated-state feedback, and
%  3) Actual state when using measured-state feedback.
xEst_Save = zeros(numStates,numSteps); % Estimated State
xESFB_Save = zeros(numStates,numSteps); % Actual, estFdbk
xMSFB_Save = zeros(numStates,numSteps); % Actual, measFdbk

% Save control, u, for both estimated-state and measured-state feedback.
uEst_Save = zeros(numSteps,1);
yEst_Save = zeros(numSteps,1);
yESFB_Save = zeros(numSteps,1);
uMSFB_Save = zeros(numSteps,1);
yMSFB_Save = zeros(numSteps,1);

% Initial Cond: x[0] = (I-(Ad-Bd*Kd))^{-1}*Bd*Krd*y_desired(end)
% xEst_Next: observer-based estimate of x[i+1].
% x_EstState_Fdbk_Next: actual x[i+1] when using estimated-state feedback
% x_MeasState_Fdbk_Next: actual x[i+1] when using measured-state feedback
xEstStateFdbk = (eye(size(Ad))-(Ad-Bd*Kd))\(Bd*Krd*y_desired(end));
xMeasStateFdbk = xEstStateFdbk; 
xEst = 0.0*xEstStateFdbk;  % Initial Estimate with an Error.

for i = 1:numSteps
    
    % Get the current desired output
    yd = y_desired(i);  

    % Inject a perturbation, zero xEst at i = 3/4*numsteps
    if perturb && i > floor(3*numSteps/4)  && i < 20 + floor(3*numSteps/4)
        xEst = 0.0*xEst;
    end
    
    % Use measured state and y_desired to compute control
    uEst = Krd*yd - Kd*xEst; % Control using estimated x.
    uMSFB = Krd*yd - Kd*(xMeasStateFdbk+NoiseAmp.*(rand(numStates,1)-0.5)); 
  
    % Compute estimated y and measured y when using estimate-state fdbk. 
    yEst = Cd*xEst; % Estimated y using estimated state.
    xEstStateFdbkN = xEstStateFdbk + NoiseAmp.*(rand(numStates,1)-0.5);
    yESFB = Cd*xEstStateFdbkN;
    xMeasStateFdbkN = xMeasStateFdbk + NoiseAmp.*(rand(numStates,1)-0.5);
    yMSFB = Cd*xMeasStateFdbkN;
    
    % Save current state and control for plotting.
    uEst_Save(i) = uEst;
    xEst_Save(:,i) = xEst; 
    yEst_Save(i) = yEst;
    xESFB_Save(:,i) = xEstStateFdbkN;
    yESFB_Save(i) = yESFB;
    
    uMSFB_Save(i) = uMSFB;
    xMSFB_Save(:,i) = xMeasStateFdbkN;
    yMSFB_Save(i) = yMSFB;

    
    % Update state for three cases:
    % 1) Estimated state corrected by difference in y's
    xEst = Ad*xEst + Bd*uEst + Ld*(yESFB-yEst) + Bi*yd;
    % 2) "Actual" next state for estimated-state feedback.
    xEstStateFdbk = Ad*xEstStateFdbk + Bd*uEst + Bi*yd;
    % 3) "Actual" next state for measured-state feedback.
    xMeasStateFdbk = Ad*xMeasStateFdbk + Bd*uMSFB + Bi*yd;

end


% Plots comparing state estimates to actual states for estimated-state 
% feedback control, alongside states for measured-state feedback control.
figure('Name', 'Comparing Estimated- and Measured-State Feedback');
numCols =2;
numRows = numStates+2;

subplot(numRows,numCols,1);

% First, plot the output, y and compare to y_desired.
plot(t,yMSFB_Save, 'bl', t, y_desired, 'k.');
axis tight;
title('Meas-State Fdbk: y_d(black) y(blue)');
curLim = axis;
subplot(numRows,numCols,2);
plot(t,yESFB_Save, 'bl', t, yEst_Save,'r', t, y_desired, 'k.');
axis(curLim);
title('Est-State Fdbk: y_d(black), yMeas(blue), yEst(red)');

% Now plot all the states.
for i = 1:numStates
    subplot(numRows,numCols,2*i+1);
    plot(t, xMSFB_Save(i,:),'bl');
    axis([0,t(end),-1.5,1.5]);
    title(['Meas-State Fdbk ',stateNames{i}]);
    curLim = axis;
    subplot(numRows,numCols,2*i+2);
    plot(t, xESFB_Save(i,:),'bl',t,xEst_Save(i,:),'r');
    axis(curLim);
    title(['actual ', stateNames{i}, '(blue)', '   estimated ',stateNames{i}, '(red)']);
end
subplot(numRows,numCols,2*(numStates+1)+1)
plot(t, uMSFB_Save,'bl');
axis('tight');% axis([0,t(end),-2.6,2.6]);
title('Meas-State u = Kr*yd-K*xMeas');
subplot(numRows,numCols,2*(numStates+1)+2)
plot(t, uEst_Save,'bl');
axis([0,t(end),-0.6,0.6]);
title('Est-State u = Kr*yd-K*xEst');


