function sys = propModel()
% This function generates a "sys" data structure (in matlab, type "doc dss"), for a 
% third-order continuous-time single-input-single-output state-space model of 
% the propellor arm system. The model is described below using the "descriptor" format:
% E dx/dt = A x + B u,
% y = Cx + Du,
% where 
% E: an invertible 3x3 matrix and is often diagonal, 
% A: is a 3x3 matrix (not necessarily invertible), 
% B is a 3x1 matrix, and
% C is a 1x3 matrix.
% 
% The States for the linearized state-space model are: 
% [Angle (Theta_a), Rotational velocity (Omega_a), back-EMF (deltaBemf)]
% My tweaks
tweakJm = 1.0;   % FIX THESE!!
tweakPropM = 1.0;

% Parameters measured experimentally.
Vss = 5;
Vsrt = sqrt(Vss);
Ke = Vsrt*1.1e-3;  % back-emf per radian/sec normalized to Vss
Km = Vsrt*1.1e-3;  % Torque per amp normalized to Vss
Jm = tweakJm*1.5e-6;  % Propeller-motor moment of inertia
Rtotal = 0.825; % Motor DC resistance + driver resistance
La = 0.15;   % Arm length in meters
PropM = tweakPropM*0.0125; % Mass of the propeller+motor
Ja = PropM*La^2; % Arm moment of inertia
Kf = Vsrt*10e-6;  % Motor Friction normalized to Vss
Kt = Vsrt*13e-4; % Thrust per radians/sec

% E matrix, multiplies dx/dt
E = zeros(3);
% A matrix, multiplies x
A = zeros(3);
% B matrix multiples u
B = zeros(3,1);
% C matrix multiplies x to produce y.
C = zeros(1,3);

% Arm rotation rate: d/dt theta = Omega.
E(1,1) = 1.0;
A(1,2) = 1;

% Arm accel: Ja d/dt omega = La*(Kt/Ke) Vbemf
E(2,2) = Ja;
A(2,3) = Kt*La/Ke;

% Prop Accel: Jm d/dt Vbemf = -Kf*Vbemf +(Ke*Km/Rtotal)*(Vpwm-Vbemf)
% Input u is Vpwm.
E(3,3) = Jm;
A(3,3) = -(Ke*Km/Rtotal+ Kf);
B(3) = Ke*Km/Rtotal;

% Output is angle sensor voltage, approximately one volt per radian.
C(1) = 1;

% We are not using the instantaneous u-to-y term.
D = 0;

% State names for labeling graphs.
stateNames = {'Arm Angle', 'Arm Rot. Vel', 'vBemf'};

sys = dss(A, B, C, D, E, 'StateName', stateNames);
end

